
#ifndef __fprintd_marshal_MARSHAL_H__
#define __fprintd_marshal_MARSHAL_H__

#include	<glib-object.h>

G_BEGIN_DECLS

/* VOID:STRING,BOOLEAN (fprintd-marshal.list:1) */
extern void fprintd_marshal_VOID__STRING_BOOLEAN (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
                                                  const GValue *param_values,
                                                  gpointer      invocation_hint,
                                                  gpointer      marshal_data);

G_END_DECLS

#endif /* __fprintd_marshal_MARSHAL_H__ */

